@@ -22,6 +22,7 @@
 
 #include <glib.h>
 #include <stdlib.h>
+#include <unistd.h>
 #include <gtk/gtk.h>
 #include <pthread.h>
 #include "main.h"
@@ -65,7 +66,16 @@ item_check(gpointer data)
   /* Check if primary contents were lost */
   if ((primary_temp == NULL) && (primary_text != NULL))
   {
-    gtk_clipboard_set_text(primary, primary_text, -1);
+    /* Check contents */
+    gint count;
+    GdkAtom *targets;
+    gboolean contents = gtk_clipboard_wait_for_targets(primary, &targets, &count);
+    g_free(targets);
+    /* Only recover lost contents if there isn't any other type of content in the clipboard */
+    if (!contents)
+    {
+      gtk_clipboard_set_text(primary, primary_text, -1);
+    }
   }
   else
   {
@@ -102,7 +112,17 @@ item_check(gpointer data)
   /* Check if clipboard contents were lost */
   if ((clipboard_temp == NULL) && (clipboard_text != NULL))
   {
-    gtk_clipboard_set_text(clipboard, clipboard_text, -1);
+    /* Check contents */
+    gint count;
+    GdkAtom *targets;
+    gboolean contents = gtk_clipboard_wait_for_targets(primary, &targets, &count);
+    g_free(targets);
+		/* Only recover lost contents if there isn't any other type of content in the clipboard */
+		if (!contents)
+		{
+      g_print("Clipboard is null, recovering ...\n");
+      gtk_clipboard_set_text(clipboard, clipboard_text, -1);
+    }
   }
   else
   {
@@ -177,23 +197,49 @@ execute_action(void *command)
   pthread_exit(NULL);
 }
 
+/* Called when execution action exits */
+static void
+action_exit(GPid pid, gint status, gpointer data)
+{
+  g_spawn_close_pid(pid);
+  if (!prefs.no_icon)
+  {
+    gtk_status_icon_set_from_stock((GtkStatusIcon*)status_icon, GTK_STOCK_PASTE);
+    gtk_status_icon_set_tooltip((GtkStatusIcon*)status_icon, _("Clipboard Manager"));
+  }
+  actions_lock = FALSE;
+}
+
 /* Called when an action is selected from actions menu */
 static void
 action_selected(GtkButton *button, gpointer user_data)
 {
-  /* Insert clipboard into command (user_data), execute and free */
+  /* Change icon and enable lock */
+  actions_lock = TRUE;
+  if (!prefs.no_icon)
+  {
+    gtk_status_icon_set_from_stock((GtkStatusIcon*)status_icon, GTK_STOCK_EXECUTE);
+    gtk_status_icon_set_tooltip((GtkStatusIcon*)status_icon, _("Executing action..."));
+  }
+  
+  /* Insert clipboard into command (user_data), and prepare it for execution */
   gchar* clipboard_text = gtk_clipboard_wait_for_text(clipboard);
   gchar* command = g_markup_printf_escaped((gchar*)user_data, clipboard_text);
   g_free(clipboard_text);
   g_free(user_data);
+  gchar* shell_command = g_shell_quote(command);
+  g_free(command);
+  gchar* cmd = g_strconcat("/bin/sh -c ", shell_command, NULL);
+  g_free(shell_command);
   
-  /* Create thread */
-  pthread_t action_thread;
-  if (pthread_create(&action_thread, NULL, execute_action, (void *)command))
-  {
-    g_free(command);
-    g_warning(_("Could not create thread for executed action\n"));
-  }
+  /* Execute action */
+  GPid pid;
+  gchar **argv;
+  g_shell_parse_argv(cmd, NULL, &argv, NULL);
+  g_free(cmd);
+  g_spawn_async(NULL, argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, &pid, NULL);
+  g_child_watch_add(pid, (GChildWatchFunc)action_exit, NULL);
+  g_strfreev(argv);
 }
 
 /* Called when Edit Actions is selected from actions menu */
@@ -289,6 +335,14 @@ clear_selected(GtkMenuItem *menu_item, gpointer user_data)
       g_slist_free(history);
       history = NULL;
       save_history();
+      g_free(primary_text);
+      g_free(clipboard_text);
+      g_free(synchronized_text);
+      primary_text = g_strdup("");
+      clipboard_text = g_strdup("");
+      synchronized_text = g_strdup("");
+      gtk_clipboard_set_text(primary, "", -1);
+      gtk_clipboard_set_text(clipboard, "", -1);
     }
     gtk_widget_destroy(confirm_dialog);
   }
@@ -298,6 +352,14 @@ clear_selected(GtkMenuItem *menu_item, gpointer user_data)
     g_slist_free(history);
     history = NULL;
     save_history();
+    g_free(primary_text);
+    g_free(clipboard_text);
+    g_free(synchronized_text);
+    primary_text = g_strdup("");
+    clipboard_text = g_strdup("");
+    synchronized_text = g_strdup("");
+    gtk_clipboard_set_text(primary, "", -1);
+    gtk_clipboard_set_text(clipboard, "", -1);
   }
 }
 
@@ -341,14 +403,23 @@ show_about_dialog(GtkMenuItem *menu_item, gpointer user_data)
     gtk_about_dialog_set_copyright((GtkAboutDialog*)about_dialog, "Copyright (C) 2007, 2008 Gilberto \"Xyhthyx\" Miralla");
     gtk_about_dialog_set_authors((GtkAboutDialog*)about_dialog, authors);
     gtk_about_dialog_set_translator_credits ((GtkAboutDialog*)about_dialog,
-                                             "Davide Truffa <davide@catoblepa.org>\n"
+                                             "Miloš Koutný <milos.koutny@gmail.com>\n"
+                                             "Kim Jensen <reklamepost@energimail.dk>\n"
                                              "Eckhard M. Jäger <bart@neeneenee.de>\n"
-                                             "Gultyaev Alexey <hokum83@gmail.com>\n"
                                              "Michael Stempin <mstempin@web.de>\n"
+                                             "Benjamin 'sphax3d' Danon <sphax3d@gmail.com>\n"
                                              "Németh Tamás <ntomasz@vipmail.hu>\n"
-                                             "Hasan Yılmaz <iletisim@hasanyilmaz.net>\n"
+                                             "Davide Truffa <davide@catoblepa.org>\n"
                                              "Jiro Kawada <jiro.kawada@gmail.com>\n"
+                                             "Øyvind Sæther <oyvinds@everdot.org>\n"
+                                             "pankamyk <pankamyk@o2.pl>\n"
+                                             "Tomasz Rusek <tomek.rusek@gmail.com>\n"
                                              "Phantom X <megaphantomx@bol.com.br>\n"
+                                             "Ovidiu D. Niţan <ov1d1u@sblug.ro>\n"
+                                             "Alexander Kazancev <kazancas@mandriva.ru>\n"
+                                             "Daniel Nylander <po@danielnylander.se>\n"
+                                             "Hedef Türkçe <iletisim@hedefturkce.com>\n"
+                                             "Lyman Li <lymanrb@gmail.com>\n"
                                              "Gilberto \"Xyhthyx\" Miralla <xyhthyx@gmail.com>");
     
     gtk_about_dialog_set_license((GtkAboutDialog*)about_dialog, license);
@@ -526,6 +597,15 @@ show_history_menu(gpointer data)
     for (element = history; element != NULL; element = element->next)
     {
       GString* string = g_string_new((gchar*)element->data);
+      /* Remove control characters */
+      int i = 0;
+      while (i < string->len)
+      {
+        if (string->str[i] == '\n')
+          g_string_erase(string, i, 1);
+        else
+          i++;
+      }
       /* Ellipsize text */
       if (string->len > prefs.item_length)
       {
@@ -555,13 +635,13 @@ show_history_menu(gpointer data)
       gtk_label_set_single_line_mode((GtkLabel*)item_label, prefs.single_line);
       
       /* Check if item is also clipboard text and make bold */
-      if ((clipboard_text) && (g_strcmp0((gchar*)element->data, clipboard_text) == 0))
+      if ((clipboard_temp) && (g_strcmp0((gchar*)element->data, clipboard_temp) == 0))
       {
         gchar* bold_text = g_markup_printf_escaped("<b>%s</b>", string->str);
         gtk_label_set_markup((GtkLabel*)item_label, bold_text);
         g_free(bold_text);
       }
-      else if ((primary_text) && (g_strcmp0((gchar*)element->data, primary_text) == 0))
+      else if ((primary_temp) && (g_strcmp0((gchar*)element->data, primary_temp) == 0))
       {
         gchar* italic_text = g_markup_printf_escaped("<i>%s</i>", string->str);
         gtk_label_set_markup((GtkLabel*)item_label, italic_text);
@@ -576,6 +656,7 @@ show_history_menu(gpointer data)
       else
         element_number++;
     }
+    /* Cleanup */
     g_free(primary_temp);
     g_free(clipboard_temp);
     /* Return history to normal if reversed */
